Crate midnight_circuits

Crate midnight_circuits 

Halo2 gadgets implemented for Midnight.

Re-exports

pub use midnight_proofs;

Modules

biguint
Big unsigned integer arithmetic.
ecc
Module that implements foreign and native EC arithmetic
field
Native and non-native arithmetic
hash
Implementations for in-circuit hashes
instructions
Set of instructions interfaces.
map
Succinct Key-Value Map Representation Using Merkle Trees
parsing
Module implementing chips and gadgets to process JSON objects.
testing_utils
Tools useful for testing
types
Types for assigned circuit values and non-assigned counterparts, and traits for treating with them generically.
vec
Vector module. This modules contains the types and gadgets to handle variable-length vectors. This type allow the length of the vector to be a witness.
verifier
In-circuit KZG-based PLONK verifier.

Traits

CircuitField
A prime field suitable for use in a circuit, as the native field or emulated.
ComposableChip
Provides a common interface for layering chips with shared resources.