pub struct AssignedNativePoint<C: CircuitCurve> { /* private fields */ }Expand description
A twisted Edwards curve point represented in affine (x, y) coordinates, the
identity represented as (0, 1).
The represented point may or may not lie in the prime order subgroup. If
in_subgroup is true, then the point has been constrained to be in the
prime order subgroup.
Since in most use cases we want to ensure the point is in the subgroup,
the implementation of InnerValue and AssignmentInstructions require
subgroup membership. To use arbitrary points of the curve there are
equivalent functions that explicitly state the absence of this subgroup
membership check.