AssignedBit

Struct AssignedBit 

pub struct AssignedBit<F: CircuitField>(/* private fields */);
This wrapper type on AssignedNative<F> is designed to enforce type safety on assigned bits. It prevents the user from creating an AssignedBit without using the designated entry points, which guarantee (with constraints) that the assigned value is indeed 0 or 1.