pub struct AssignedByte<F: CircuitField>(/* private fields */);Expand description
This wrapper type on AssignedNative<F> is designed to enforce type safety
on assigned bytes. It prevents the user from creating an AssignedByte
without using the designated entry points, which guarantee (with
constraints) that the assigned value is indeed in the range [0, 256).