ParserGadget

Struct ParserGadget 

pub struct ParserGadget<F, N>{ /* private fields */ }
A gadget for parsing json data. It is parametrized by:

  • F: the native field,
  • N: a set of in-circuit native instructions.

Implementations

Source

impl<F, N> ParserGadget<F, N>

pub fn ascii_to_int( &self, layouter: &mut impl Layouter<F>, input: &[AssignedByte<F>], ) -> Result<AssignedNative<F>, Error>

Given a string of ASCII digits as assigned bytes, returns the represented integer as an assigned native. Leading 0s are allowed.

Unsatisfiable Circuit

If the input bytes are not valid ASCII digits, i.e. in the range [48, 57].

Panics

If |input| exceeds the maximum number of digits that can be reliably stored in an F element.

pub fn date_to_int( &self, layouter: &mut impl Layouter<F>, input: &[AssignedByte<F>], format: (DateFormat, Separator), century_base: Option<&AssignedNative<F>>, ) -> Result<AssignedNative<F>, Error>

Given an ASCII string of assigned bytes, representing a date in the specified format, returns DD + 100 * MM + 10_000 * YYYY as an assigned native value. This function does not check the validity of the date (e.g. "32011990" in DDMMYYYY is accepted as 32 January 1990).

For two-digit year formats (DateFormat::YYMMDD), century_base must be Some(N) where N is an assigned value in [0, 99]. The year is then resolved as 1900 + YY + (if YY < N { 100 } else { 0 }), i.e. the 100-year window is [1900+N, 2000+N). The caller is responsible for constraining N < 100.

For four-digit year formats, century_base must be None.

Source

impl<F, N> ParserGadget<F, N>

pub fn assert_date_before_fixed( &self, layouter: &mut impl Layouter<F>, date_bytes: &[AssignedByte<F>], format: (DateFormat, Separator), limit_date: Date, ) -> Result<(), Error>

Asserts that a date (as assigned bytes in the given format) is strictly before limit_date. Convenience wrapper around date_to_int + a range check.

Source

impl<F, N> ParserGadget<F, N>

pub fn new(native_gadget: &N) -> Self

Create a new parser gadget.

pub fn fetch_bytes( &self, layouter: &mut impl Layouter<F>, sequence: &[AssignedByte<F>], idx: &AssignedNative<F>, len: usize, ) -> Result<Vec<AssignedByte<F>>, Error>

Given a sequence of assigned bytes, an index idx (represented with an assigned native value) and a length len, returns a vector of length len that is guaranteed to contain consecutive bytes from sequence starting at idx. Namely: vec![sequence[idx], sequence[idx+1]..., sequence[idx+len-1]].

This is enforced with constraints while keeping idx private.

Unsatisfiable Circuit

If idx is not in the range [0, |sequence| - len].