Module vec

Module vec 

Vector module. This modules contains the types and gadgets to handle variable-length vectors. This type allow the length of the vector to be a witness.

Re-exports

pub use vector::*;

Modules

vector
Module containing basic vector types and utilities.
vector_gadget
Gadget for handling vectors of elements that fit in the native field: Native elements, bytes and bits.