Module vector

Module vector 

Module containing basic vector types and utilities.

Structs

AssignedVector
A variable-length vector of elements of type T, with size bound M.

Traits

Vectorizable
Trait for the individual elements of an AssignedVector.

Functions

get_lims
Returns the range where the data should be placed in the buffer.