pub struct AssignedMsm<S: SelfEmulation> { /* private fields */ }Expand description
Type for in-circuit multi-scalar multiplications.
This is the in-circuit analog of Msm<C>.
Implementations
Sourceimpl<S: SelfEmulation> AssignedMsm<S>
impl<S: SelfEmulation> AssignedMsm<S>
Sourcepub fn as_public_input_with_committed_scalars(
msm: &Msm<S>,
) -> (Vec<S::F>, Vec<S::F>)
pub fn as_public_input_with_committed_scalars( msm: &Msm<S>, ) -> (Vec<S::F>, Vec<S::F>)
Converts the off-circuit MSM into two vectors of scalars. The first will be used as a normal instance, whereas the second will be plugged-in in as a committed instance.
The committed instance part corresponds to the (fixed and non-fixed) scalars of the MSM.
Sourceimpl<S: SelfEmulation> AssignedMsm<S>
impl<S: SelfEmulation> AssignedMsm<S>
Sourcepub fn assign(
layouter: &mut impl Layouter<S::F>,
curve_chip: &S::CurveChip,
scalar_chip: &S::ScalarChip,
len: usize,
fixed_base_names: &[String],
msm_value: Value<Msm<S>>,
) -> Result<Self, Error>
pub fn assign( layouter: &mut impl Layouter<S::F>, curve_chip: &S::CurveChip, scalar_chip: &S::ScalarChip, len: usize, fixed_base_names: &[String], msm_value: Value<Msm<S>>, ) -> Result<Self, Error>
Witnesses an MSM computation of len bases/scalars and a BTreeMap of
fixed_base_scalars indexed by the given fixed_base_names.
Warning
The points of the MSM are not enforced to be part of the relevant prime order subgroup.
Panics
If msm_value is known and its number of variable bases differs from
len, or its number of fixed-base scalars differs from
fixed_base_names.len().
Sourcepub fn empty() -> Self
pub fn empty() -> Self
An empty AssignedMsm with no fixed base scalars, that evaluates to the identity point.
Sourcepub fn from_term(
scalar: &AssignedBoundedScalar<S::F>,
base: &S::AssignedPoint,
) -> Self
pub fn from_term( scalar: &AssignedBoundedScalar<S::F>, base: &S::AssignedPoint, ) -> Self
Creates a new MSM from the given base (with a scalar of 1).
Sourcepub fn from_fixed_term(
scalar: &AssignedBoundedScalar<S::F>,
base_name: &str,
) -> Self
pub fn from_fixed_term( scalar: &AssignedBoundedScalar<S::F>, base_name: &str, ) -> Self
Creates a new MSM from the given fixed base name (with a scalar of 1).
Sourcepub fn add_term(
&mut self,
scalar: &AssignedBoundedScalar<S::F>,
base: &S::AssignedPoint,
)
pub fn add_term( &mut self, scalar: &AssignedBoundedScalar<S::F>, base: &S::AssignedPoint, )
Adds a (scalar, base) term to the AssignedMsm.
Sourcepub fn add_msm(
&mut self,
layouter: &mut impl Layouter<S::F>,
scalar_chip: &S::ScalarChip,
other: &Self,
) -> Result<(), Error>
pub fn add_msm( &mut self, layouter: &mut impl Layouter<S::F>, scalar_chip: &S::ScalarChip, other: &Self, ) -> Result<(), Error>
Adds two AssignedMsm.
Sourcepub fn collapse(
&mut self,
layouter: &mut impl Layouter<S::F>,
curve_chip: &S::CurveChip,
scalar_chip: &S::ScalarChip,
) -> Result<(), Error>
pub fn collapse( &mut self, layouter: &mut impl Layouter<S::F>, curve_chip: &S::CurveChip, scalar_chip: &S::ScalarChip, ) -> Result<(), Error>
Evaluates the variable part of the AssignedMsm (the scalar-base pairs) collapsing it to a single point (and a scalar of 1), leaving the fixed-base part intact.
This function mutates self.
Sourcepub fn resolve_fixed_bases(
&mut self,
fixed_bases: &BTreeMap<String, S::AssignedPoint>,
)
pub fn resolve_fixed_bases( &mut self, fixed_bases: &BTreeMap<String, S::AssignedPoint>, )
Given the actual fixed bases, resolves the fixed-base part of the MSM by pairing each named scalar with its base and moving them to regular variable-base entries.
After this call, fixed_base_scalars becomes empty.
Panics
If some of the keys in fixed_base_scalars do not appear in the
provided fixed_bases map.
Sourcepub fn scale(
&mut self,
layouter: &mut impl Layouter<S::F>,
scalar_chip: &S::ScalarChip,
r: &AssignedBoundedScalar<S::F>,
) -> Result<(), Error>
pub fn scale( &mut self, layouter: &mut impl Layouter<S::F>, scalar_chip: &S::ScalarChip, r: &AssignedBoundedScalar<S::F>, ) -> Result<(), Error>
Scales all the scalars of the AssignedMsm by the given factor r.
This function mutates self.
Sourcepub fn accumulate_with_r(
&self,
layouter: &mut impl Layouter<S::F>,
scalar_chip: &S::ScalarChip,
other: &Self,
r: &AssignedBoundedScalar<S::F>,
) -> Result<Self, Error>
pub fn accumulate_with_r( &self, layouter: &mut impl Layouter<S::F>, scalar_chip: &S::ScalarChip, other: &Self, r: &AssignedBoundedScalar<S::F>, ) -> Result<Self, Error>
Accumulates two AssignedMSMs with the a given scalar r.
The resulting AssignedMSMs evaluates (on fixed_bases) to
self.eval(fixed_bases) + r * other.eval(fixed_bases).