Module containing basic vector types and utilities.
Structs
- Assigned
Vector - A variable-length vector of elements of type T, with size bound M.
Traits
- Vectorizable
- Trait for the individual elements of an AssignedVector.
Functions
- get_
lims - Returns the range where the data should be placed in the buffer.