pub struct AssignedVector<F: CircuitField, T: Vectorizable, const M: usize, const A: usize> { /* private fields */ }Expand description
A variable-length vector of elements of type T, with size bound M.
lenis the (potentially secret) effective length of the vector, its value is guaranteed to be in the range[0, M].bufferis the padded payload of this vector; it contains the effective data of the vector as well as filler values, which are UNCONSTRAINED.
The effective payload in the data is aligned in A sized chunks. This enables more efficient implementations of instructions like hashing over this type. As a result of this alignment, the data may contain filler values before and after the effective payload. The padding in front of the payload will always be 0 mod A, so that the payload begins aligned in A sized chunks. The padding at the end of the payload will have a size in [0, A) such that | front_pad | + | payload | + | back_pad | = M.
Invariant: M must be a multiple of A (and A > 0, M >= A).
Several operations (padding_flag, get_limits, hashing, …) rely on this
to guarantee that the buffer decomposes into exactly M / A full chunks
and that a full-capacity vector (len == M) can always be placed without
overflow. This is enforced by the entry points
assign_with_filler and
assign.